/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved.
 *
 * This program and the accompanying materials are made
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/

package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;

import com.trackplus.track.ws.beans.WSBaseOptionsContainer;
import com.trackplus.track.ws.beans.WSLabelValueBean;
import com.trackplus.track.ws.beans.WSOptionBean;
import com.trackplus.track.ws.beans.WSSortedOptionBean;
import com.trackplus.track.ws.beans.WSStateOptionBean;
import com.trackplus.track.ws.beans.WSTreeNode;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

/**
 * <p>
 * In some input forms you may define fields where the user can select from a
 * number of options, like status, priority, manager or responsible.
 * </p>
 * <p>
 * This class demonstrates how to retrieve all available option fields and their
 * possible values via the web service interface.
 * </p>
 * <p>
 * The class uses the {@link LoginSample} class to perform the login. You need
 * to set the URL and the user name and password in that class before you
 * compile and run this class here.
 * </p>
 * <p>
 * This example is a Java console application. You can run it directly by
 * calling its <code>main()</code> method.
 * </p>
 */

public class BaseOptionSample extends LoginSample {
	public BaseOptionSample() {
		super();
	}

	public BaseOptionSample(TCLFacade connector) {
		super(connector);
	}

	/**
	 * Obtains the projects from the server
	 *
	 */
	public void getProjectsIssueTypes() {
		System.out.println("\n->BaseOptionSample.getProjectsIssueTypes()");
		try {
			WSLabelValueBean[] projects = connector.getProjects();
			if (projects == null || projects.length == 0) {
				System.out.println("No projects found on the server. Maybe the user does not have right in any project");
			} else {
				System.out.println("Successfully obtained " + projects.length + " projects:");
				for (int i = 0; i < projects.length; i++) {
					String projectID = projects[i].getValue();
					System.out.println("Project: ID=" + projectID + " Label=" + projects[i].getLabel());
					WSLabelValueBean[] issueTypes = connector.getIssueTypes(projectID);
					for (int j = 0; j < issueTypes.length; j++) {
						System.out.println("\t IssueType: ID=" + issueTypes[j].getValue() + " Label=" + issueTypes[j].getLabel());
					}
					System.out.println();
				}
				System.out.println();
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("<-BaseOptionSample.getProjectsIssueTypes()\n");
	}

	/**
	 * Getting the base options: all projects; all issueTypes; all priorities;
	 * all severities; all states;
	 */
	public void getBaseOptionsContainer() {
		System.out.println("\n->BaseOptionSample.getBaseOptionsContainer()");
		try {
			WSBaseOptionsContainer baseOptionsContainer = connector.getBaseOptionsContainer();
			// all projects
			WSOptionBean[] allProjects = baseOptionsContainer.getProjects();
			printOptions("Project", allProjects);
			System.out.println();

			// all issue types
			WSSortedOptionBean[] allIssueTypes = baseOptionsContainer.getIssueTypes();
			printSortedOptions("IssueType", allIssueTypes);
			System.out.println();

			// all priorities
			WSSortedOptionBean[] allPriorities = baseOptionsContainer.getPriorities();
			printSortedOptions("Priority", allPriorities);
			System.out.println();

			// all severities
			WSSortedOptionBean[] allSeverities = baseOptionsContainer.getSeverities();
			printSortedOptions("Severity", allSeverities);
			System.out.println();

			// all states
			WSStateOptionBean[] allStates = baseOptionsContainer.getStates();
			printStateOptions("State", allStates);
			System.out.println();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("<-BaseOptionSample.getBaseOptionsContainer()\n");
	}

	private void printOptions(String optionType, WSOptionBean[] options) {
		for (int i = 0; i < options.length; i++) {
			System.out.println(optionType + ": ID=" + options[i].getObjectID() + " Label=" + options[i].getLabel());
		}
	}

	private void printSortedOptions(String optionType, WSSortedOptionBean[] options) {
		for (int i = 0; i < options.length; i++) {
			System.out.println(optionType + ": ID=" + options[i].getObjectID() + " Label=" + options[i].getLabel() + " SortOrder="
					+ options[i].getSortOrderValue());
		}
	}

	private void printStateOptions(String optionType, WSStateOptionBean[] options) {
		for (int i = 0; i < options.length; i++) {
			System.out.println(optionType + ": ID=" + options[i].getObjectID() + " Label=" + options[i].getLabel() + " SortOrder="
					+ options[i].getSortOrderValue() + " StrateFlag=" + options[i].getStateFlag());
		}
	}

	public void getProjectsAsTree() {
		System.out.println("\n->BaseOptionSample.getProjectsAsTree()");
		try {
			WSTreeNode[] projects = connector.getProjectsAsTree();
			if (projects == null || projects.length == 0) {
				System.out.println("No projects found on the server. Maybe the user does not have right in any project");
			} else {
				System.out.println("Successfully obtained " + projects.length + " projects:");
				for (int i = 0; i < projects.length; i++) {
					printNode(projects[i], "");
				}
				System.out.println();
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("<-BaseOptionSample.getProjectsAsTree()\n");
	}

	
	public static void printNode(WSTreeNode node, String indent) {
		System.out.println(indent + "[id:" + node.getId() + ", label=" + node.getLabel() + ", icon=" + node.getIcon() + "]");
		WSTreeNode[] children = node.getChildren();
		if (children != null && children.length > 0) {
			for (int i = 0; i < children.length; i++) {
				printNode(children[i], indent + "\t");
			}
		}
	}

	public static void main(String[] args) {

		BaseOptionSample baseOptionSample = new BaseOptionSample();

		baseOptionSample.login(uName,pField);

		baseOptionSample.login();

		baseOptionSample.getProjectsAsTree();

		// get projects and issueTypes
		baseOptionSample.getProjectsIssueTypes();

		// base drop-down container
		// baseOptionSample.getBaseOptionsContainer();
		// baseOptionSample.getProjectsIssueTypesAndSubprojects();
	}
}
